from sqlalchemy import Column, create_engine, func, ForeignKey
from sqlalchemy.dialects.postgresql import UUID, VARCHAR, TIMESTAMP, SMALLINT, JSONB, TEXT
from sqlalchemy.ext.declarative import declarative_base
from uuid import uuid4

Base = declarative_base()

class Workers(Base):
    __tablename__ = "workers"

    uuid = Column(UUID(as_uuid=True),primary_key=True,default=uuid4)
    persona = Column(VARCHAR(12))
    name = Column(VARCHAR(50))
    last_name = Column(VARCHAR(50))
    email = Column(VARCHAR(150))
    username = Column(VARCHAR(30))
    status = Column(SMALLINT)
    logged_at = Column(TIMESTAMP(timezone=True))
    session_deadline = Column(TIMESTAMP(timezone=True))
    created_at = Column(TIMESTAMP(timezone=True),server_default=func.current_timestamp())
    updated_at = Column(TIMESTAMP(timezone=True),onupdate=func.current_timestamp())
    session_id = Column(VARCHAR(50))

class WorkerHistory(Base):
    __tablename__ = "worker_history"

    uuid = Column(UUID(as_uuid=True),primary_key=True,default=uuid4)
    worker_uuid = Column(UUID(as_uuid=True),ForeignKey(Workers.uuid), nullable=False)
    status_code = Column(VARCHAR(20))
    status = Column(VARCHAR(20))
    reason = Column(TEXT)
    created_at = Column(TIMESTAMP(timezone=True),server_default=func.current_timestamp())
    updated_at = Column(TIMESTAMP(timezone=True),onupdate=func.current_timestamp())

class Tasks(Base):
    __tablename__ = "tasks"

    uuid = Column(UUID(as_uuid=True),primary_key=True,default=uuid4)
    worker_uuid = Column(UUID(as_uuid=True),ForeignKey(Workers.uuid), nullable=False)
    name = Column(VARCHAR(150))
    data = Column(JSONB)
    status_code = Column(VARCHAR(5))
    status = Column(VARCHAR(20))
    priority = Column(SMALLINT)
    deadline = Column(TIMESTAMP(timezone=True))
    requested_at = Column(TIMESTAMP(timezone=True))
    created_at = Column(TIMESTAMP(timezone=True),server_default=func.current_timestamp())
    updated_at = Column(TIMESTAMP(timezone=True),onupdate=func.current_timestamp())

class TaskHistory(Base):
    __tablename__ = "task_history"

    uuid = Column(UUID(as_uuid=True),primary_key=True,default=uuid4)
    task_uuid = Column(UUID(as_uuid=True),ForeignKey(Tasks.uuid), nullable=False)
    worker_uuid = Column(UUID(as_uuid=True),ForeignKey(Workers.uuid), nullable=False)
    data = Column(JSONB)
    reason = Column(TEXT)
    status_code = Column(VARCHAR(5))
    status = Column(VARCHAR(20))
    created_at = Column(TIMESTAMP(timezone=True),server_default=func.current_timestamp())
    updated_at = Column(TIMESTAMP(timezone=True),onupdate=func.current_timestamp())

# if __name__ == "__main__":
#     engine = create_engine("postgresql://DB_USER:DB_PASS@DB_HOST:DB_PORT/DB_NAME")
#     Base.metadata.create_all(bind=engine)